//
//  SceneDelegate.swift
//  SPMTest2
//
//  Created by Catalina on 8/9/19.
//  Copyright © 2019 Designated Nerd Software. All rights reserved.
//

import UIKit
import Apollo
import ApolloSQLite
import ApolloWebSocket

class SceneDelegate: UIResponder, UIWindowSceneDelegate {

    var window: UIWindow?
    
    lazy var url = URL(string: "http://localhost:8080/graphql")!
    lazy var wsURL = URL(string: "ws://localhost:8080/websocket")!
    lazy var transport: SplitNetworkTransport = {
        let http = HTTPNetworkTransport(url: self.url)
        let request = URLRequest(url: self.wsURL)
        let webSocket = WebSocketTransport(request: request)
        return SplitNetworkTransport(httpNetworkTransport: http,
                                     webSocketNetworkTransport: webSocket)
    }()
    
    lazy var storeURL: URL = {
        let docs = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first!
        let documentsURL = URL(fileURLWithPath: docs)
        let storeURL = documentsURL.appendingPathComponent("db.sqlite")
        return storeURL
    }()

    lazy var cache = try! SQLiteNormalizedCache(fileURL: self.storeURL)
    lazy var store = ApolloStore(cache: self.cache)
    
    lazy var client = ApolloClient(networkTransport: self.transport,
                                   store: self.store)

    func scene(_ scene: UIScene, willConnectTo session: UISceneSession, options connectionOptions: UIScene.ConnectionOptions) {
        // Use this method to optionally configure and attach the UIWindow `window` to the provided UIWindowScene `scene`.
        // If using a storyboard, the `window` property will automatically be initialized and attached to the scene.
        // This delegate does not imply the connecting scene or session are new (see `application:configurationForConnectingSceneSession` instead).
        guard let _ = (scene as? UIWindowScene) else { return }
    }

    func sceneDidDisconnect(_ scene: UIScene) {
        // Called as the scene is being released by the system.
        // This occurs shortly after the scene enters the background, or when its session is discarded.
        // Release any resources associated with this scene that can be re-created the next time the scene connects.
        // The scene may re-connect later, as its session was not neccessarily discarded (see `application:didDiscardSceneSessions` instead).
    }

    func sceneDidBecomeActive(_ scene: UIScene) {
        // Called when the scene has moved from an inactive state to an active state.
        // Use this method to restart any tasks that were paused (or not yet started) when the scene was inactive.
        
        self.client.fetch(query: HeroNameQuery(), cachePolicy: .returnCacheDataAndFetch) { result in
            switch result {
            case .success(let graphQLResult):
                print("Hero name: \(String(describing: graphQLResult.data?.hero?.name))")
            case .failure(let error):
                print("ERROR: \(error)")
            }
            
        }
        
        self.client.subscribe(subscription: ReviewAddedSubscription()) { result in
            switch result {
            case .success(let graphQLResult):
                print("Review added: \(String(describing: graphQLResult.data?.reviewAdded))")
            case .failure(let error):
                print("SUBSCRIPTION ERROR: \(error)")
            }
        }
        
        self.createReviewsEveryFewSeconds()
    }
    
    func createReviewsEveryFewSeconds() {
        Timer.scheduledTimer(withTimeInterval: 2, repeats: true) { _ in
            self.createReview()
        }
    }
    
    func createReview() {
        print("Creating review!")
        self.client.perform(mutation: CreateAwesomeReviewMutation())
    }

    func sceneWillResignActive(_ scene: UIScene) {
        // Called when the scene will move from an active state to an inactive state.
        // This may occur due to temporary interruptions (ex. an incoming phone call).
    }

    func sceneWillEnterForeground(_ scene: UIScene) {
        // Called as the scene transitions from the background to the foreground.
        // Use this method to undo the changes made on entering the background.
    }

    func sceneDidEnterBackground(_ scene: UIScene) {
        // Called as the scene transitions from the foreground to the background.
        // Use this method to save data, release shared resources, and store enough scene-specific state information
        // to restore the scene back to its current state.
    }


}

